/******************************************************************************
* File name:
*   Contact.java
* Author:
*   Barham, Ray
* Creation Date:
*   06/10/07
* Developed on:
*   WinXP
* Developed with:
*   Java(TM) SE Development Kit 6
*   JCreator 4.00.028
*
* Description:
*   Instances of this class will contain all the needed contact information
*   for an individual in the MyChat program.  It is set as final to keep
*   someone from extending it and possibly causing a security risk.
******************************************************************************/

/******************************************************************************
* Import statements
******************************************************************************/
import java.io.Serializable;

final public class Contact implements Serializable
{
	/**************************************************************************
	* Data Members:
	*   String name = Contains the name of the contact.
	*   String host = Contains the host/IP of the contact.
	**************************************************************************/
	private String name;
	private String host;
	
	/**************************************************************************
	* Method Name:
	*   Contact
	* Arguments Passed:
	*   None
	* Data Returned:
	*   None
	* Method Description:
	*  Constructs a blank Contact object.
	**************************************************************************/
	public Contact()
	{
		name = "";
		host = "";
	}
	
	/**************************************************************************
	* Method Name:
	*   Contact
	* Arguments Passed:
	*   String name = Name of the contact.
	*   String host = The host/ip of the contact.
	* Data Returned:
	*   None
	* Method Description:
	*  Constructs a Contact object with the provided name and host/ip
	**************************************************************************/
	public Contact(String name, String host)
	{
		this.name = name;
		this.host = host;
	}
	
	/**************************************************************************
	* Method Name:
	*   Contact
	* Arguments Passed:
	*   Contact contact = The Contact object to copy
	* Data Returned:
	*   None
	* Method Description:
	*  Copies the contents from the provided Contact object to a new Contact
	*  object.
	**************************************************************************/
	public Contact(Contact contact)
	{
		this.name = contact.getName();
		this.host = contact.getHost();
	}
	
	/**************************************************************************
	* Method Name:
	*   getName
	* Arguments Passed:
	*   None
	* Data Returned:
	*   String = The name of the contact
	* Method Description:
	*  Returns the name of the contact.
	**************************************************************************/
	public String getName()
	{
		return name;
	}
	
	/**************************************************************************
	* Method Name:
	*   getHost
	* Arguments Passed:
	*   None
	* Data Returned:
	*   String = The host/ip of the contact
	* Method Description:
	*  Returns the host/ip of the contact.
	**************************************************************************/
	public String getHost()
	{
		return host;
	}
	
	/**************************************************************************
	* Method Name:
	*   setName
	* Arguments Passed:
	*   String name = The new name of the contact.
	* Data Returned:
	*   None
	* Method Description:
	*   Sets the name of the contact with the name provided.
	**************************************************************************/
	public void setName(String name)
	{
		this.name = name;
	}
	
	/**************************************************************************
	* Method Name:
	*   setHost
	* Arguments Passed:
	*   String host = The new host/ip of the contact.
	* Data Returned:
	*   None
	* Method Description:
	*   Sets the host/ip of the contact with the host/ip provided.
	**************************************************************************/
	public void setHost(String host)
	{
		this.host = host;
	}
	
	/**************************************************************************
	* Method Name:
	*   toString
	* Arguments Passed:
	*   None
	* Data Returned:
	*   String = A string representation of this object.
	* Method Description:
	*   Returns a string representation of this object.
	**************************************************************************/
	public String toString()
	{
		return getName() + "@" + getHost();
	}
}