/******************************************************************************
* File name:
*   MyChatSSL.java
* Author:
*   Barham, Ray
* Creation Date:
*   5/29/2007
* Developed on:
*   WinXP
* Developed with:
*   Java(TM) SE Development Kit 6
*   JCreator 4.00.028
*
* Description:
*   MyChatSSL is a peer-to-peer chat program that uses Secure Socket Layer 
*   (SSL) to provide secure communication between clients.  This version 
*   isnt too different then the original version, MyChat, except for SSL 
*   support.  Please download the readme from www.burningcode.com in the
*   portfolio secion for more info.
*
* Compile and Run:
*
*   Before compiling or running, ensure that the java/bin directory is in you 
*   systems PATH, unless you want to do things the long way. :-).  See 
*   http://java.sun.com/docs/books/tutorial/essential/environment/paths.html 
*   for information on how to do this.
*
*   To compile into bytecode:
*
*      Browse to the directory of the source code and type
*      javac MyChatSSL.java
*
*   To run the bytecode:
*
*      Browse to the directory where the bytecode is and type
*      java MyChatSSL
*
*   To run the bytecode with SSL debug on:
*
*      Browse to the directory where the bytecode is and type
*      java -Djavax.net.debug=ssl MyChatSSL
*
*   To run the jar file (Note: To run the jar file download, you will need Java 6.):
*
*      Browse to the directory where the .jar file is located and type or double click
*      java jar MyChatSSL.jar
*
* TODO:
*   Create UI components to change to connecting and listening port.
*   Change m_jta_MessageList from a jtextarea to a jtextpane (to display colors).
*   Add ability to send a file.
*   Redesign UI to be better handled.
*   Add the ability to clear the message list.
*   Fix a problem with setting the caret position to the end of the message
*     list when receiving a long message.
*   Add an audio notification of new message and the ability to turn it off.
*
* Credits:
*   Tony Sintes:
*   Thank you for your wonderful tutorial "Mr. Happy Object teaches custom events"
*   http://www.javaworld.com/javaworld/javaqa/2002-03/01-qa-0315-happyevent.html
*
*   Todd Sundsted:
*   Thank you for the awesome info on SSL and Java
*   http://www.javaworld.com/javaworld/jw-05-2001/jw-0511-howto.html?page=1
*
*   Todd Sundsted:
*   Thank you for the great howto on SSL and peer-to-peer.
*   http://www.ibm.com/developerworks/java/library/j-p2pssl/
******************************************************************************/

/******************************************************************************
* Import statements
******************************************************************************/
// IO import statements
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;

// AWT import statements
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

// Util import statements
import java.util.Arrays;
import java.util.Enumeration;

// Network import statements
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.UnknownHostException;
import javax.net.ssl.KeyManager;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.SSLServerSocket;
import javax.net.ssl.SSLServerSocketFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManagerFactory;

// Security import statements
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;

// Swing import statements
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

final public class MyChatSSL extends JFrame implements ActionListener
{
	/**************************************************************************
	* Data Members:
	*   JTextArea m_jta_MessageList = Text area that lists all the messages sent or received.
	*   JTextField m_jtf_Message = Text field were the message to be sent is typed.
	*   JList m_jl_RecipientList = List of recipients to send messages to. Can also be thought of as a favorites list.
	*   JButton m_jb_SendButton = Button pressed to send the message in m_jtf_Message.
	*   JButton m_jb_AddButton = Button pressed to add a recipient to m_jl_RecipientList.
	*   JButton m_jb_EditButton = Button pressed to edit a recipient from m_jl_RecipientList.
	*   JButton m_jb_DelButton = Button pressed to delete a recipient from m_jl_RecipientList.
	*   JMenuItem m_jmi_ChangeUserInfo = Menu item used for changing the user information.
	*   JCheckBoxMenuItem m_jcbmi_AutoAddNewRecipients = Check box used for enabling or disabling automatically adding new recipients.
	*   DefaultListModel m_dlm_Recipients = List model containing Contact objects for all the recipients.
	*   Contact m_MyInfo = the object containing user information.  It is accessed via m_jmi_ChangeUserInfo.
	*   String m_s_SettingsFile = the file that contains the settings.
	*   String m_s_RecipientFile = the csv file that contains the recipient list.
	*   int m_i_Port = the port to send and receive messages on.
	**************************************************************************/
	private JTextArea m_jta_MessageList;
	private JTextField m_jtf_Message;
	private JList m_jl_RecipientList;
	private JButton	m_jb_SendButton;
	private JButton m_jb_AddButton;
	private JButton m_jb_EditButton;
	private JButton m_jb_DelButton;
	private JMenuItem m_jmi_ChangeUserInfo;
	private JCheckBoxMenuItem m_jcbmi_AutoAddNewRecipients;
	private DefaultListModel m_dlm_Recipients = new DefaultListModel();
	private Contact m_MyInfo = new Contact();
	private String m_s_SettingsFile = "settings.dat";
	private String m_s_RecipientFile = "list.csv";
	private int m_i_Port = 10099;

	/**************************************************************************
	* Method Name:
	*   MyChat
	* Arguments Passed:
	*   String title = The title of the frame
	*   int width = The width of the frame
	*   int height = The height of the frame
	* Data Returned:
	*   None
	* Method Description:
	*  Constructs a new MyChat frame with the specified title, width and 
	*  height.
	**************************************************************************/
	public MyChatSSL(String title, int width, int height)
	{
		/*
		 *  Setup the frame
		 */
		super(title);
		JPanel contentPane = new JPanel(new BorderLayout());
		WindowSetup.centerWindow(this, width, height);
		setMinimumSize(new Dimension(width, height));
		setContentPane(contentPane);
		addWindowListener(new WindowAdapter()
		{
			/*
			 *  Called when the frame is initially opened
			 */
			public void windowOpened(WindowEvent e)
			{
				/*
				 *  Read in and populate the settings, if not exist then first time use is assumed.
				 */
				if(!readSettings(m_s_SettingsFile))
				{
					MyDialog.showDialog(
						(JFrame)e.getWindow(),
						"Welcome to " + ((JFrame)e.getWindow()).getTitle() + ".\nPlease fill out the following form before continuing.",
						"Welcome to " + ((JFrame)e.getWindow()).getTitle(),
						true,
						"images/funny.png",
						MyDialog.OK_OPTION);
					
					try
					{
						m_MyInfo = new Contact("", InetAddress.getLocalHost().getHostName().toLowerCase());
						
						while(m_MyInfo.getName().isEmpty() || m_MyInfo.getHost().isEmpty())
							RecipientDialog.showDialog((Frame)e.getWindow(), "Set User Info", true, m_MyInfo);
	
						if(!writeSettings(m_s_SettingsFile))
							MyDialog.showWarningDialog((JFrame)e.getWindow(), "Error saving settings to file.", true);
					}
					
					catch(UnknownHostException uhe)
					{
						MyDialog.showWarningDialog(
							(JFrame)e.getWindow(), 
							"Error getting host name for this computer\n" + uhe.getMessage(),
							true);
					}
				}
				
				/*
				 *  Read in and populate the recipients list, if the file exist
				 */
				readRecipients(m_s_RecipientFile);
			}
			
			/*
			 *  Called when the frame is closing
			 */
			public void windowClosing(WindowEvent e)
			{
				/*
			 	 *  Write the setting to file
			 	 */
				if(!writeSettings(m_s_SettingsFile))
					MyDialog.showWarningDialog(null, "Error saving settings to file.", true);
			}
		});
		
		/*
		 *  Create and setup the menu bar items
		 */
		m_jmi_ChangeUserInfo = new JMenuItem("Change User Info");
		m_jmi_ChangeUserInfo.setMnemonic(KeyEvent.VK_C);
		m_jmi_ChangeUserInfo.setToolTipText("Click to change your information");
		m_jmi_ChangeUserInfo.addActionListener(this);

		m_jcbmi_AutoAddNewRecipients = new JCheckBoxMenuItem("Auto Add New Recipients");
		m_jcbmi_AutoAddNewRecipients.setMnemonic(KeyEvent.VK_A);
		m_jcbmi_AutoAddNewRecipients.setSelected(true);
		m_jcbmi_AutoAddNewRecipients.setToolTipText("Click to disable automatically adding of new recipients");
		m_jcbmi_AutoAddNewRecipients.addActionListener(this);
		
		/*
		 *  Create and setup the menus.  Add the menu items to the menus
		 */
		JMenu settingsMenu = new JMenu("Settings");
		settingsMenu.setMnemonic(KeyEvent.VK_S);
		settingsMenu.add(m_jmi_ChangeUserInfo);
		settingsMenu.add(m_jcbmi_AutoAddNewRecipients);
		
		/*
		 *  Create and setup the menu bar.  Add the menus to the menu bar
		 */
		JMenuBar menuBar = new JMenuBar();
		menuBar.add(settingsMenu);
		setJMenuBar(menuBar);
		
		/*
		 *  Create and setup the message list text area
		 */
		m_jta_MessageList = new JTextArea();
		m_jta_MessageList.setEditable(false);
		m_jta_MessageList.setWrapStyleWord(true);
		m_jta_MessageList.setLineWrap(true);
		m_jta_MessageList.setForeground(Color.blue);
		m_jta_MessageList.setBackground(Color.white);
		m_jta_MessageList.getDocument().addDocumentListener(new DocumentListener()
		{
			/*
			 *  Not used but required to implement.
			 */
			public void changedUpdate(DocumentEvent e){}
			
			/*
			 *  Called when the message list has a message inserted. 
			 */
			public void insertUpdate(DocumentEvent e)
			{
				// Set the position in the message list to the bottom
				m_jta_MessageList.setCaretPosition(e.getDocument().getLength());
			}
			
			/*
			 *  Not used but required to implement.
			 */
			public void removeUpdate(DocumentEvent e){}
		});
		
		/*
		 *  Create and setup the recipients list and list model
		 */
		m_jl_RecipientList = new JList(m_dlm_Recipients);
		m_jl_RecipientList.setFixedCellWidth(120);
		m_jl_RecipientList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		m_dlm_Recipients.addListDataListener(new ListDataListener()
		{
			/*
		 	 *  Write the recipient list to file when a recipient in the recipient
		 	 *  list model changes.
		 	 */
			public void contentsChanged(ListDataEvent e)
			{
				if(!writeRecipients(m_s_RecipientFile))
					MyDialog.showWarningDialog(null, "Error saving recipients to file.", true);
			}
			
			/*
		 	 *  Write the recipient list to file when a recipient in the recipient
		 	 *  list model has been added.
		 	 */
			public void intervalAdded(ListDataEvent e)
			{
				if(!writeRecipients(m_s_RecipientFile))
					MyDialog.showWarningDialog(null, "Error saving recipients to file.", true);
			}
			
			/*
		 	 *  Write the recipient list to file when a recipient in the recipient
		 	 *  list model removed.
		 	 */
			public void intervalRemoved(ListDataEvent e)
			{
				if(!writeRecipients(m_s_RecipientFile))
					MyDialog.showWarningDialog(null, "Error saving recipients to file.", true);
			}
		});

		/*
		 *  Create and setup the message to send text field
		 */
		m_jtf_Message = new JTextField();
		m_jtf_Message.setForeground(Color.blue);
		m_jtf_Message.setBackground(Color.white);
		m_jtf_Message.addActionListener(this);
		
		/* 
		 *  Create and setup the buttons
		 */
		m_jb_SendButton = new JButton("Send");
		m_jb_SendButton.addActionListener(this);
		
		m_jb_AddButton = new JButton("Add");
		m_jb_AddButton.addActionListener(this);
		
		m_jb_EditButton = new JButton("Edit");
		m_jb_EditButton.addActionListener(this);
		
		m_jb_DelButton = new JButton("Delete");
		m_jb_DelButton.addActionListener(this);
		
		/*
		 *  Create, setup, and add components to the message list panel
		 */
		JPanel jpMessageListPanel = new JPanel(new BorderLayout());
		jpMessageListPanel.setBorder(new TitledBorder("Message List"));
		jpMessageListPanel.add(new JScrollPane(m_jta_MessageList), BorderLayout.CENTER);
		
		/*
		 *  Create, setup, and add components to the recipient button panel
		 */
		JPanel jpRecipientButtonPanel = new JPanel(new GridLayout(3, 1));
		jpRecipientButtonPanel.add(m_jb_AddButton);
		jpRecipientButtonPanel.add(m_jb_EditButton);
		jpRecipientButtonPanel.add(m_jb_DelButton);
		
		/*
		 *  Create, setup, and add componetns to the recipients panel
		 */
		JPanel jpRecipientsPanel = new JPanel(new BorderLayout());					
		jpRecipientsPanel.setBorder(new TitledBorder("Recipients"));
		jpRecipientsPanel.add(new JScrollPane(m_jl_RecipientList), BorderLayout.CENTER);
		jpRecipientsPanel.add(jpRecipientButtonPanel, BorderLayout.SOUTH);
		

		/*
		 *  Create, setup, and add components to the message sending panel
		 */
		JPanel jpMessagePanel = new JPanel(new BorderLayout());
		jpMessagePanel.setBorder(new TitledBorder("Send Message"));
		jpMessagePanel.add(m_jtf_Message, BorderLayout.CENTER);
		jpMessagePanel.add(m_jb_SendButton, BorderLayout.EAST);
		
		/*
		 *  Add the panels to the content pane panel
		 */
		contentPane.add(jpRecipientsPanel, BorderLayout.EAST);
		contentPane.add(jpMessageListPanel, BorderLayout.CENTER);
		contentPane.add(jpMessagePanel, BorderLayout.SOUTH);
	}
	
	/**************************************************************************
	* Method Name:
	*   actionPerformed
	* Arguments Passed:
	*   ActionEvent event = An object that refrences the performed event
	* Data Returned:
	*   None
	* Method Description:
	*   This method handles ActionListener events.
	**************************************************************************/
	public void actionPerformed(ActionEvent event)
	{
		/*
		 *  If the send button was pressed or enter was pressed in the message field
		 */
		if(event.getSource() == m_jb_SendButton || event.getSource() == m_jtf_Message)
			sendButtonPress();
		
		/*
		 *  If the add recipient button was pressed
		 */
		else if(event.getSource() == m_jb_AddButton)
			addButtonPress();
		
		/*
		 *  If the edit recipient button was pressed
		 */
		else if(event.getSource() == m_jb_EditButton)
			editButtonPress();
		
		/*
		 *  If the delete recipient button was pressed
		 */
		else if(event.getSource() == m_jb_DelButton)
			delButtonPress();
		
		/*
		 *  If a menu item was pressed
		 */
		else if(event.getSource() instanceof JMenuItem)
			menuItemPress((JMenuItem)event.getSource());
	}

	/**************************************************************************
	* Method Name:
	*   sendButtonPress
	* Arguments Passed:
	*   None
	* Data Returned:
	*   None
	* Method Description:
	*   Called when the send button is pressed or enter was hit in the message
	*   field.
	**************************************************************************/
	private void sendButtonPress()
	{
		/*
		 *  Ensure that a recipient is selected then send construct the message
		 *  to send.
		 */
		if(!m_jl_RecipientList.isSelectionEmpty())
		{
			String message = m_jtf_Message.getText();
			Contact client = (Contact)m_jl_RecipientList.getSelectedValue();
			
			m_jta_MessageList.append("To [" + client.getName() + "]: " + message + "\n");
			m_jtf_Message.setText("");
			
			sendMessage(client.getHost(), m_i_Port, message);
		}
		
		else
			MyDialog.showMessageDialog(this, "Please select a recipient.", false);
	}

	/**************************************************************************
	* Method Name:
	*   addButtonPress
	* Arguments Passed:
	*   None
	* Data Returned:
	*   None
	* Method Description:
	*   Called when the add recipient button is pressed.
	**************************************************************************/
	private void addButtonPress()
	{
		RecipientDialog.addRecipient(this, false, m_dlm_Recipients);
	}
	
	/**************************************************************************
	* Method Name:
	*   editButtonPress
	* Arguments Passed:
	*   None
	* Data Returned:
	*   None
	* Method Description:
	*   Called when the edit recipient button is pressed.
	**************************************************************************/
	private void editButtonPress()
	{
		/*
		 *  Ensure that a recipient is selected before trying to edit it
		 */
		if(!m_jl_RecipientList.isSelectionEmpty())
		{
			RecipientDialog.editRecipient(
				this,
				false,
				(Contact)m_jl_RecipientList.getSelectedValue(),
				m_dlm_Recipients);
		}

		else
			MyDialog.showMessageDialog(this, "Please select a recipient.", false);
	}
	
	/**************************************************************************
	* Method Name:
	*   delButtonPress
	* Arguments Passed:
	*   None
	* Data Returned:
	*   None
	* Method Description:
	*   Called when the delete recipient button is pressed.
	**************************************************************************/
	private void delButtonPress()
	{
		/*
		 *  Ensure that a recipient is selected before trying to delete it
		 */
		if(!m_jl_RecipientList.isSelectionEmpty())
			m_dlm_Recipients.removeElement((Contact)m_jl_RecipientList.getSelectedValue());

		else
			MyDialog.showMessageDialog(this, "Please select a recipient.", false);
	}

	/**************************************************************************
	* Method Name:
	*   menuItemPress
	* Arguments Passed:
	*   JMenuItem menuItem = The JMenuItem object that was pressed.
	* Data Returned:
	*   None
	* Method Description:
	*   Called when a item from the menu bar is pressed.
	**************************************************************************/
	private void menuItemPress(JMenuItem menuItem)
	{
		/*
		 *  If the Change User Info menu item was pressed.
		 */
		if(menuItem == m_jmi_ChangeUserInfo)
			RecipientDialog.showDialog(this, "Change User Info", false, m_MyInfo);
		
		/*
		 *  If the Auto Add New Recipients check box was pressed.  All this really does
		 *  is change the tooltip when it is activated.
		 */
		else if(menuItem == m_jcbmi_AutoAddNewRecipients)
		{
			if(m_jcbmi_AutoAddNewRecipients.isSelected())
				m_jcbmi_AutoAddNewRecipients.setToolTipText("Click to disable automatically adding of new recipients");
			
			else
				m_jcbmi_AutoAddNewRecipients.setToolTipText("Click to enable automatically adding of new recipients");
		}
	}

	/**************************************************************************
	* Method Name:
	*   sendMessage
	* Arguments Passed:
	*   String message = Message to send to the server
	* Data Returned:
	*   None
	* Method Description:
	*   Handles the sending of the messages to the recipients.
	**************************************************************************/
	private void sendMessage(String client, int port, final String message)
	{
		try
		{
			/*
			 *  Connect to a recipient.
			 */
			SSLSocket socketToServer = (SSLSocket)SSLSocketFactory.getDefault().createSocket(client, port);
			socketToServer.setEnabledCipherSuites(socketToServer.getSupportedCipherSuites());
	
			socketToServer.startHandshake();

			if(socketToServer.isConnected())
			{
				/*
				 *  Get the output stream from the recipient and wrap it around a PrintWriter
				 *  and ObjectOutputStream for easy processing.
				 */
				PrintWriter outputPrintWriter = new PrintWriter(socketToServer.getOutputStream(), true);
				ObjectOutputStream outputObject = new ObjectOutputStream(socketToServer.getOutputStream());
				
				/*
				 *  Send this clients user information and the message to the recipient
				 */
				outputObject.writeObject(m_MyInfo);
				outputPrintWriter.println(message);

				outputPrintWriter.close();
				outputObject.close();
			}
			
			else
				MyDialog.showWarningDialog(this, "Connection failed.", false);

		}
		
		catch(IOException e)
		{
			MyDialog.showWarningDialog(this, "Error sending message\n" + e.getMessage(), false);
		}
	}
	
	/**************************************************************************
	* Method Name:
	*   receiveMessages
	* Arguments Passed:
	*   None
	* Data Returned:
	*   None
	* Method Description:
	*   Receives messages from recipients send this client messages.  It is 
	*   not called during the creation of this object to make it possible 
	*   to compile a version that only sends messages, not receive them.
	**************************************************************************/
	public void receiveMessages()
	{
		/*
		 *  This loop is needed if an error occurs during the creation of the
		 *  SSLServerSocket.  If an error does occur (see the proceding "try, catch"
		 *  statement), then the SSL keystore and truststore settings will be
		 *  cleared and the default settings will be used.
		 */
		while(true)
		{
			try
			{
				/*
				 *  Create a server socket and give it a the port number to listen on.
				 */
				SSLServerSocket serverSocket = (SSLServerSocket)SSLServerSocketFactory.getDefault().createServerSocket(m_i_Port);
				serverSocket.setEnabledCipherSuites(serverSocket.getSupportedCipherSuites());
	
				while(true)
				{
					try
					{
						serverSocket.setEnabledCipherSuites(serverSocket.getSupportedCipherSuites());
						/*
						 *  When a recipient connects, create a socket.  serverSocket.accept() will 
						 *  wait until a client has connected before continuing.
						 */
						SSLSocket socket = (SSLSocket)serverSocket.accept();
	
						/*
					 	 *  Get the input stream from the recipient and wrap it around a PrintWriter
					 	 *  and ObjectOutputStream for easy processing.
					 	 */
						BufferedReader inputBufferedReader = new BufferedReader(new InputStreamReader(socket.getInputStream()));
						ObjectInputStream inputObject = new ObjectInputStream(socket.getInputStream());
						
						/*
					 	 *  Get this recipients information.
					 	 */
						Contact contact = (Contact)inputObject.readObject();
						Contact existingContact = getRecipient(contact);
			
						/*
						 *  If the ability is enabled, check to see if the recipient is already in 
						 *  the list and, if they aren't, add them.
						 */
						if(m_jcbmi_AutoAddNewRecipients.isSelected() && existingContact == null)
							RecipientDialog.addRecipient(contact, m_dlm_Recipients);
	
						/*
						 *  Get the recipients message and put it in the message list.
						 */
						String message = inputBufferedReader.readLine();
						
						if(existingContact == null)
							m_jta_MessageList.append("From [" + contact.getName() + "]: " + message + "\n");
						else
							m_jta_MessageList.append("From [" + existingContact.getName() + "]: " + message + "\n");
						
						inputBufferedReader.close();
						inputObject.close();
					}
					
					catch(IOException e)
					{
						MyDialog.showWarningDialog(this, "Error receiving message\n" + e.getMessage(), false);
					}
					
					catch(ClassNotFoundException e)
					{
						MyDialog.showWarningDialog(this, "Error receiving object\n" + e.getMessage(), false);
					}
				}
			}
	
			catch(IOException e)
			{
				MyDialog.showWarningDialog(
					this,
					"Error setting up server connection.\n" + e.getMessage() + "\nClearing keystore and truststore information and using default parameters.",
					true);
				
				/*
				 *  Clear out the keystore and truststore system settings.
				 */
				System.setProperty("javax.net.ssl.keyStore", "");
				System.setProperty("javax.net.ssl.keyStorePassword", "");
				System.setProperty("javax.net.ssl.trustStore", "");
				System.setProperty("javax.net.ssl.trustStorePassword", "");
			}
		}
	}
	
	/**************************************************************************
	* Method Name:
	*   getRecipient
	* Arguments Passed:
	*   Contact contact = contact to look for.
	* Data Returned:
	*   Contact = The contact with the same host name, null if not found.
	* Method Description:
	*   Checks to see if a contact is already in the recipient list.
	**************************************************************************/
	private Contact getRecipient(Contact contact)
	{
		/*
		 *  Loop through all the recipients and compare their host to that of the
		 *  passed contact.
		 */
		for(Enumeration enumeration = m_dlm_Recipients.elements(); enumeration.hasMoreElements();)
		{
			Contact existingContact = (Contact)enumeration.nextElement();					

			if(contact.getHost().compareToIgnoreCase(existingContact.getHost()) == 0)
				return existingContact;
		}
		
		return null;
	}
	
	/**************************************************************************
	* Method Name:
	*   readRecipients
	* Arguments Passed:
	*   String file = The CSV file to read the recipients from
	* Data Returned:
	*   boolean = True if read was successful or file not found.  False if
	*     error reading from the file.
	* Method Description:
	*   Reads in all the recipients from the passed file (if it
	*   exists) and stores it in the recipient list DefaultListModel.
	* CSV file format:
	*   username,[hostname|IP]
	**************************************************************************/
	private boolean readRecipients(String file)
	{
		try
		{
			// Open the file and wrap it in a BufferedReader object
 			BufferedReader inputBufferedReader = new BufferedReader(new FileReader(file));
			
			String inputLine;

			/*
		 	 *  Cycle through all the recipients in the file and store them in 
		 	 *  the list model
		 	 */
			while((inputLine = inputBufferedReader.readLine()) != null)
			{
				String name = inputLine.substring(0, inputLine.indexOf(","));
				String host = inputLine.substring(inputLine.indexOf(",")+1).trim().toLowerCase();

				m_dlm_Recipients.addElement(new Contact(name, host));
			}
			
			inputBufferedReader.close();
			
			return true;
		}
		
		catch(FileNotFoundException e)
		{
			return false;
		}
		
		catch(IOException e)
		{
			return false;
		}
	}
	
	/**************************************************************************
	* Method Name:
	*   writeRecipients
	* Arguments Passed:
	*   String file = The CSV file to write the recipients to.
	* Data Returned:
	*   boolean = True if write was successful or file not found.  False if
	*     error reading from the file.
	* Method Description:
	*   This method writes all the recipients to the passed file from 
	*   recipient list model.
	* CSV file format:
	*   username,[hostname|IP]
	**************************************************************************/
	private boolean writeRecipients(String file)
	{
		try
		{
			// Create/Overwrite a file and wrap it in a PrintWriter object 
 			PrintWriter outputPrintWriter = new PrintWriter(new FileWriter(file));

			/*
			 *  Cycle through the recipient list model and store each element to the file
			 */
			for(Enumeration enumeration = m_dlm_Recipients.elements(); enumeration.hasMoreElements();)
			{
				Contact contact = (Contact)enumeration.nextElement();
				
				outputPrintWriter.println(contact.getName() + "," + contact.getHost());
			}
			
			outputPrintWriter.close();
			
			return true;
		}
		
		catch(FileNotFoundException e)
		{
			return false;
		}
		
		catch(IOException e)
		{
			return false;
		}
	}
	
	/**************************************************************************
	* Method Name:
	*   readSettings
	* Arguments Passed:
	*   String file = Path to the file to read from
	* Data Returned:
	*   boolean = true if read successful, false otherwise
	* Method Description:
	*   This method reads in the settings for the client from passed string
	**************************************************************************/
	private boolean readSettings(String file)
	{
		try
		{
			// Open the file and wrap it in a BufferedReader object
 			BufferedReader inputBufferedReader = new BufferedReader(new FileReader(file));
 			
			String inputLine;
			
			/*
			 *  Cycle through the settings file and apply the settings.
			 */
			while((inputLine = inputBufferedReader.readLine()) != null)
			{
				String setting = inputLine.substring(0, inputLine.indexOf("=")).toLowerCase();
				
				/*
				 *  If the current setting is the username
				 */
				if("username".equals(setting))
				{
					String value = inputLine.substring(inputLine.indexOf("=")+1);
					m_MyInfo.setName(value);
				}
				
				/*
				 *  If the current setting is the host
				 */
				else if("host".equals(setting))
				{
					String value = inputLine.substring(inputLine.indexOf("=")+1).toLowerCase().trim();
					m_MyInfo.setHost(value);
				}
				
				/*
				 *  If the current setting is the auto add new recipients
				 */
				else if("auto_add_recipient".equals(setting))
				{
					boolean value = Boolean.valueOf(inputLine.substring(inputLine.indexOf("=")+1).toLowerCase().trim());
					
					if(value)
					{
						m_jcbmi_AutoAddNewRecipients.setSelected(true);
						m_jcbmi_AutoAddNewRecipients.setToolTipText("Click to disable automatically adding of new recipients");
					}
					
					else
					{
						m_jcbmi_AutoAddNewRecipients.setSelected(false);
						m_jcbmi_AutoAddNewRecipients.setToolTipText("Click to enable automatically adding of new recipients");
					}
				}
				
				/*
				 *  If the current setting is the port number
				 */
				else if("port".equals(setting))
				{
					String value = inputLine.substring(inputLine.indexOf("=")+1).toLowerCase().trim();
					
					if(!value.equals(""))
						m_i_Port = Integer.parseInt(value);
				}
			}
			
			inputBufferedReader.close();
			
			return true;
		}
		
		catch(FileNotFoundException e)
		{
			return false;
		}
		
		catch(IOException e)
		{
			return false;
		}
	}
	
	/**************************************************************************
	* Method Name:
	*   writeSettings
	* Arguments Passed:
	*   String file = Path to the file to write to.
	* Data Returned:
	*   boolean = true if read successful, false otherwise
	* Method Description:
	*   This method write the settings for the client to the passed string
	**************************************************************************/
	private boolean writeSettings(String file)
	{
		try
		{
			// Create/Overwrite a file and wrap it in a PrintWriter object 
 			PrintWriter outputPrintWriter = new PrintWriter(new FileWriter(file));
 			
 			/*
 			 *  Write the settings to file
 			 */
 			outputPrintWriter.println("username=" + m_MyInfo.getName());
 			outputPrintWriter.println("host=" + m_MyInfo.getHost());
 			outputPrintWriter.println("auto_add_recipient=" + m_jcbmi_AutoAddNewRecipients.isSelected());
 			outputPrintWriter.println("port=" + m_i_Port);
 			
			outputPrintWriter.close();
			
			return true;
		}
		
		catch(FileNotFoundException e)
		{
			return false;
		}
		
		catch(IOException e)
		{
			return false;
		}
	}
	
	/**************************************************************************
	* Method Name:
	*   readSSLSettings
	* Arguments Passed:
	*   String file = Path to the file to read from
	* Data Returned:
	*   boolean = true if read successful, false otherwise
	* Method Description:
	*   This method reads in the SSL settings from a file.  The settings are
	*   the keystore, keystore password, truststore, and truststore password.
	*   It is not required to import any of these settings.  The cipher
	*   suite used if these settings are not provided will be anonymous.
	**************************************************************************/
	public boolean readSSLSettings(String file)
	{
		try
		{
			// Open the file and wrap it in a BufferedReader object
 			BufferedReader inputBufferedReader = new BufferedReader(new FileReader(file));
 			
			String inputLine;
			
			String keyStore = null;
			String keyStorePassword = null;
			String trustStore = null;
			String trustStorePassword = null;
			
			/*
			 *  Cycle through the SSL settings file and apply the settings.
			 */
			while((inputLine = inputBufferedReader.readLine()) != null)
			{
				String setting = inputLine.substring(0, inputLine.indexOf("=")).toLowerCase().trim();
				String value = inputLine.substring(inputLine.indexOf("=")+1);
				
				if(!value.equals(""))
				{
					/*
					 *  If the current setting is the keystore filename
					 */
					if("keystore".equals(setting))
					{
						keyStore = value;
						System.setProperty("javax.net.ssl.keyStore", keyStore);
					}
					
					/*
					 *  If the current setting is the keystore password
					 */
					else if("keystorepassword".equals(setting))
					{
						keyStorePassword = value;
						System.setProperty("javax.net.ssl.keyStorePassword", keyStorePassword);
					}
					
					/*
					 *  If the current setting is the truststore filename
					 */
					else if("truststore".equals(setting))
					{
						trustStore = value;
						System.setProperty("javax.net.ssl.trustStore", trustStore);
					}
					
					/*
					 *  If the current setting is the truststore password
					 */
					else if("truststorepassword".equals(setting))
					{
						trustStorePassword = value;
						System.setProperty("javax.net.ssl.trustStorePassword", trustStorePassword);
					}
				}
			}
			
			/*
			 *  If the keystore password was not provided and the keystore file was, then
			 *  request the password.
			 */
			if(keyStore != null && keyStorePassword == null)
			{
				JPasswordField passField = new JPasswordField();
				Object[] passObj = {
					"Enter the password for the keystore file: " + keyStore,
					passField};
				
				int option = JOptionPane.showConfirmDialog(
					this,
					passObj,
					"Password Required",
					JOptionPane.OK_CANCEL_OPTION,
					JOptionPane.INFORMATION_MESSAGE);
				
				if(option == JOptionPane.OK_OPTION)
					System.setProperty("javax.net.ssl.keyStorePassword", new String(passField.getPassword()));
			}
			
			/*
			 *  If the truststore password was not provided and the truststore file was, then
			 *  request the password.
			 */
			if(trustStore != null && trustStorePassword == null)
			{
				JPasswordField passField = new JPasswordField();
				Object[] passObj = {
					"Enter the password for the truststore file: " + trustStore,
					passField};
				
				int option = JOptionPane.showConfirmDialog(
					this,
					passObj,
					"Password Required",
					JOptionPane.OK_CANCEL_OPTION,
					JOptionPane.INFORMATION_MESSAGE);
				
				if(option == JOptionPane.OK_OPTION)
					System.setProperty("javax.net.ssl.trustStorePassword", new String(passField.getPassword()));
			}
			
			inputBufferedReader.close();
			
			return true;
		}
		
		catch(FileNotFoundException e)
		{
			try
			{
				/*
				 *  Create the SSL settings file for future reference
				 */
				PrintWriter outputPrintWriter = new PrintWriter(new FileWriter(file));
				
				/*
	 			 *  Write the settings to file
	 			 */
	 			outputPrintWriter.println("KeyStore=");
	 			outputPrintWriter.println("KeyStorePassword=");
	 			outputPrintWriter.println("TrustStore=");
	 			outputPrintWriter.println("TrustStorePassword=");
	 			
				outputPrintWriter.close();
				
				return true;
			}
			
			catch(IOException ioe)
			{
				return false;
			}
		}
		
		catch(IOException e)
		{
			return false;
		}
	}

	/**************************************************************************
	* Method Name:
	*   main
	* Arguments Passed:
	*   String[] args = The arguments being passed to the program.  None are
	*     are required at this time
	* Data Returned:
	*   None
	* Method Description:
	*   This is the starting point of the program.
	**************************************************************************/
	public static void main(String[] args)
	{
		MyChatSSL frame = new MyChatSSL("MyChatSSL", 400, 300);
		
		if(!frame.readSSLSettings("ssl_settings.dat"))
			MyDialog.showWarningDialog(null, "Error reading in SSL settings file.", true);

		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.receiveMessages();
	}
}