/******************************************************************************
* File name:
*   MyDialog.java
* Author:
*   Barham, Ray
*
* Creation Date:
*   6/13/2007
* 
* Developed on:
*   WinXP
* Developed with:
*   Java(TM) SE Development Kit 6
*   JCreator 4.00.028
*
* Description:
*   This class is used to display a variety of dialog boxes.  Very similar to
*   JOptionPane but with differend icons and the ability to change modality.
*
* Credits:
*   Images from...
*     KDE Crystal Diamond 2.6 Gentoo Mod
*       http://www.kde-look.org/content/show.php/Crystal+Diamond+Icons?content=45576
*     Friendly fire by sniffels
*       http://sniffels.deviantart.com/
******************************************************************************/

/******************************************************************************
* Import statements
******************************************************************************/
// IO import statements
import java.io.IOException;

// Net import statements
import java.net.URL;

// AWT import statements
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;

// Swing import statements
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;

// Image IO import statements
import javax.imageio.ImageIO;

final public class MyDialog extends JDialog implements ActionListener
{
	/**************************************************************************
	* Data Members:
	*   int option = The value that is to be returned from the dialog box.
	*
	*   static String m_s_CriticalIcon = Path to the icon for the critical dialog
	*   static String m_s_InfoIcon = Path to the icon for the message dialog
	*   static String m_s_QuestionIcon = Path to the icon for the question dialog
	*   static String m_s_WarningIcon = Path to the icon for the warning dialog
	*
	*   static final int OK_OPTION = Is used when specifying a single button dialog.
	*   static final int YES_NO_OPTION = Is used when specifying a Yes/No dialog.
	*   static final int YES_OPTION = Returned if the Yes button is pressed in a Yes/No dialog.
	*   static final int NO_OPTION = Returned if the No button is pressed in a Yes/No dialog.
	**************************************************************************/
	private int option;
	
	private static String m_s_CriticalIcon = "images/messagebox_critical.png";
	private static String m_s_InfoIcon = "images/messagebox_info.png";
	private static String m_s_QuestionIcon = "images/messagebox_question.png";
	private static String m_s_WarningIcon = "images/messagebox_warning.png";
	
	public static int OK_OPTION = 0;
	public static int YES_NO_OPTION = 1;
	public static int YES_OPTION = 2;
	public static int NO_OPTION = 3;

	/**************************************************************************
	* Method Name:
	*   MyDialog
	* Arguments Passed:
	*   Frame owner = The owner of the dialog.  Null if none
	*   String message = The message to display in the dialog
	*   String title = The title of the dialog
	*   boolean modal = True to block user input to other to-level windows,
	*     false otherwise.
	*   String iconFile = The icon to display in the dialog
	*   int dialogType = The dialog type.  Possible values...
	*     MyDialog.OK_OPTION
	*     MyDialog.YES_NO_OPTION;
	*     MyDialog.YES_OPTION;
	*     MyDialog.NO_OPTION;
	* Data Returned:
	*   None
	* Method Description:
	*   Constructor that is used when creating a dialog box that is to have a
	*   Frame object as the owner.
	**************************************************************************/
	public MyDialog(Frame owner, String message, String title, boolean modal, String iconFile, int dialogType)
	{
		super(owner, modal);
		createDialog(message, title, modal, iconFile, dialogType);
	}
	
	/**************************************************************************
	* Method Name:
	*   MyDialog
	* Arguments Passed:
	*   Dialog owner = The owner of the dialog.  Null if none
	*   String message = The message to display in the dialog
	*   String title = The title of the dialog
	*   boolean modal = True to block user input to other to-level windows,
	*     false otherwise.
	*   String iconFile = The icon to display in the dialog
	*   int dialogType = The dialog type.  Possible values...
	*     MyDialog.OK_OPTION
	*     MyDialog.YES_NO_OPTION;
	*     MyDialog.YES_OPTION;
	*     MyDialog.NO_OPTION;
	* Data Returned:
	*   None
	* Method Description:
	*   Constructor that is used when creating a dialog box that is to have a
	*   Dialog object as the owner.
	**************************************************************************/
	public MyDialog(Dialog owner, String message, String title, boolean modal, String iconFile, int dialogType)
	{
		super(owner, modal);
		createDialog(message, title, modal, iconFile, dialogType);
	}
	
	/**************************************************************************
	* Method Name:
	*   createDialog
	* Arguments Passed:
	*   String message = Message to display on the dialog
	*   String title = Title of the dialog
	*   boolean modal = The modality of the dialog.  See Dialog.ModalityType
	*     for more info
	*   String iconFile = The icon to display in the dialog
	*   int dialogType = The dialog type.  Possible values...
	*     MyDialog.OK_OPTION
	*     MyDialog.YES_NO_OPTION;
	*     MyDialog.YES_OPTION;
	*     MyDialog.NO_OPTION;
	* Data Returned:
	*   None
	* Method Description:
	*   Performs all the work in creating a universal MyDialog dialog box.  It
	*   is set to private to force the use of creating an instance of the
	*   dialog manually or using one of the provided static methods.
	**************************************************************************/
	private void createDialog(String message, String title, boolean modal, String iconFile, int dialogType)
	{
		setOption(dialogType);
		
		URL imageURL = getClass().getResource(iconFile);
		
		ImageIcon image = null;
		
		if(imageURL != null)
			image = new ImageIcon(imageURL);

		/*
		 *  Setup the content pane
		 */
		JPanel contentPane = new JPanel(new BorderLayout());
		
		/*
		 *  Setup the area to display the message
		 */
		JTextArea messageArea = new JTextArea(message);
		messageArea.setEditable(false);
		messageArea.setBorder(null);
		messageArea.setBackground(contentPane.getBackground());
		messageArea.setAlignmentX(Component.CENTER_ALIGNMENT);
		
		/*
		 *  Setup and add components to the central panel
		 */
		JPanel centerPanel = new JPanel();
		centerPanel.setLayout(new BoxLayout(centerPanel, BoxLayout.Y_AXIS));
		centerPanel.add(Box.createVerticalStrut(10));
		centerPanel.add(messageArea);
		
		// Add some spacing
		centerPanel.add(Box.createVerticalStrut(10));
		
		/*
		 *  If this is a yes/no dialog
		 */
		if(dialogType == YES_NO_OPTION)
		{
			if(getModalityType() == ModalityType.MODELESS)
				setModalityType(DEFAULT_MODALITY_TYPE);
			
			/*
		 	 *  Create and setup the yes button
		 	 */
			JButton yesButton = new JButton("Yes");

			URL yesImageURL = getClass().getResource("images/ok.png");

			if(yesImageURL != null) 
				yesButton.setIcon(new ImageIcon(yesImageURL));
			
			yesButton.setAlignmentX(Component.LEFT_ALIGNMENT);
			yesButton.setActionCommand("yes");
			yesButton.addActionListener(this);
			getRootPane().setDefaultButton(yesButton);
			
			/*
		 	 *  Create and setup the no button
		 	 */
			JButton noButton = new JButton("No");
			
			URL noImageURL = getClass().getResource("images/cancel.png");
			
			if(noImageURL != null)
				noButton.setIcon(new ImageIcon(noImageURL));
			
			noButton.setAlignmentX(Component.RIGHT_ALIGNMENT);
			noButton.setActionCommand("no");
			noButton.addActionListener(this);
			
			/*
		 	 *  Setup and add components to the button panel
		 	 */
			JPanel buttonPanel = new JPanel(new FlowLayout());
			buttonPanel.add(yesButton);
			buttonPanel.add(noButton);
			centerPanel.add(buttonPanel);
		}
		
		else
		{
			/*
		 	 *  Create and setup the OK button
		 	 */
		 	JButton okButton = new JButton("OK");
		 	
		 	URL okImageURL = getClass().getResource("images/ok.png");
		 	
		 	if(okImageURL != null)
		 		okButton.setIcon(new ImageIcon(okImageURL));
			
			okButton.setAlignmentX(Component.CENTER_ALIGNMENT);
			okButton.setActionCommand("ok");
			okButton.addActionListener(this);
			getRootPane().setDefaultButton(okButton);
			
			// Add the OK button to the central panel
			centerPanel.add(okButton);
		}

		// Add some spacing
		centerPanel.add(Box.createVerticalStrut(10));  
		
		/*
		 *  Create and add the icon to the image panel
		 */
		JPanel imagePanel = new JPanel();
		
		if(image != null)
			imagePanel.add(new JLabel(image));

		/*
		 *  Add all the panels to the content pane
		 */
		contentPane.add(imagePanel, BorderLayout.WEST);
		contentPane.add(centerPanel, BorderLayout.CENTER);
		
		// Add some spacing on right side
		if(image != null)
			contentPane.add(Box.createHorizontalStrut(image.getIconWidth()+10), BorderLayout.EAST);
		else
			contentPane.add(Box.createHorizontalStrut(10), BorderLayout.EAST);

		/*
		 *  Setup the dialog
		 */
		if(image != null)
			setIconImage(image.getImage());

		setContentPane(contentPane);
		setResizable(false);		
		setTitle(title);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		pack();
		WindowSetup.centerWindow(this, getWidth(), getHeight());
	}
	
	/**************************************************************************
	* Method Name:
	*   actionPerformed
	* Arguments Passed:
	*   ActionEvent event = An object that refrences the performed event
	* Data Returned:
	*   None
	* Method Description:
	*   This method handles ActionListener events.
	**************************************************************************/
	public synchronized void actionPerformed(ActionEvent event)
	{
		/*
		 *  If the yes button was pressed
		 */
		if("yes".equals(event.getActionCommand()))
		{
			setVisible(false);
			setOption(YES_OPTION);
		}
		
		/*
		 *  If the no button was pressed
		 */
		else if("no".equals(event.getActionCommand()))
		{
			setVisible(false);
			setOption(NO_OPTION);
		}
		
		/*
		 *  All other events
		 */
		else
			setVisible(false);
	}

	/**************************************************************************
	* Method Name:
	*   showCriticalDialog
	* Arguments Passed:
	*   Frame owner = The owner of the dialog.  Null if none
	*   String message = The message to display in the dialog
	* Data Returned:
	*   None
	* Method Description:
	*   Creates a critical dialog with the title 'Attention!' and blocks user
	*   interaction to top-level windows.
	**************************************************************************/
	public static void showCriticalDialog(Frame owner, String message)
	{
		showDialog(owner, message, "Attention!", true, m_s_CriticalIcon, OK_OPTION);
	}
	
	/**************************************************************************
	* Method Name:
	*   showCriticalDialog
	* Arguments Passed:
	*   Frame owner = The owner of the dialog.  Null if none
	*   String message = The message to display in the dialog
	*   String title = The title of the dialog
	* Data Returned:
	*   None
	* Method Description:
	*   Creates a critical dialog that blocks user interaction to top-level 
	*   windows.
	**************************************************************************/
	public static void showCriticalDialog(Frame owner, String message, String title)
	{
		showDialog(owner, message, title, true, m_s_CriticalIcon, OK_OPTION);
	}
	
	/**************************************************************************
	* Method Name:
	*   showCriticalDialog
	* Arguments Passed:
	*   Frame owner = The owner of the dialog.  Null if none
	*   String message = The message to display in the dialog
	*   boolean modal = True to block user input to other to-level windows,
	*     false otherwise.
	* Data Returned:
	*   None
	* Method Description:
	*   Creates a critical dialog with the title 'Attention!'.
	**************************************************************************/
	public static void showCriticalDialog(Frame owner, String message, boolean modal)
	{
		showDialog(owner, message, "Attention!", modal, m_s_CriticalIcon, OK_OPTION);
	}
	
	/**************************************************************************
	* Method Name:
	*   showCriticalDialog
	* Arguments Passed:
	*   Frame owner = The owner of the dialog.  Null if none
	*   String message = The message to display in the dialog
	*   String title = The title of the dialog
	*   boolean modal = True to block user input to other to-level windows,
	*     false otherwise.
	* Data Returned:
	*   None
	* Method Description:
	*   Creates a critical dialog.
	**************************************************************************/
	public static void showCriticalDialog(Frame owner, String message, String title, boolean modal)
	{
		showDialog(owner, message, title, modal, m_s_CriticalIcon, OK_OPTION);
	}

	/**************************************************************************
	* Method Name:
	*   showMessageDialog
	* Arguments Passed:
	*   Frame owner = The owner of the dialog.  Null if none
	*   String message = The message to display in the dialog
	* Data Returned:
	*   None
	* Method Description:
	*   Creates a message dialog with the title 'Information' and blocks user
	*   interaction to top-level windows.
	**************************************************************************/
	public static void showMessageDialog(Frame owner, String message)
	{
		showDialog(owner, message, "Information", true, m_s_InfoIcon, OK_OPTION);
	}
	
	/**************************************************************************
	* Method Name:
	*   showMessageDialog
	* Arguments Passed:
	*   Frame owner = The owner of the dialog.  Null if none
	*   String message = The message to display in the dialog
	*   String title = The title of the dialog
	* Data Returned:
	*   None
	* Method Description:
	*   Creates a message dialog that blocks user interaction to top-level 
	*   windows.
	**************************************************************************/
	public static void showMessageDialog(Frame owner, String message, String title)
	{
		showDialog(owner, message, title, true, m_s_InfoIcon, OK_OPTION);
	}
	
	/**************************************************************************
	* Method Name:
	*   showMessageDialog
	* Arguments Passed:
	*   Frame owner = The owner of the dialog.  Null if none
	*   String message = The message to display in the dialog
	*   boolean modal = True to block user input to other to-level windows,
	*     false otherwise.
	* Data Returned:
	*   None
	* Method Description:
	*   Creates a message dialog with the title 'Information'.
	**************************************************************************/
	public static void showMessageDialog(Frame owner, String message, boolean modal)
	{
		showDialog(owner, message, "Information", modal, m_s_InfoIcon, OK_OPTION);
	}
	
	/**************************************************************************
	* Method Name:
	*   showMessageDialog
	* Arguments Passed:
	*   Frame owner = The owner of the dialog.  Null if none
	*   String message = The message to display in the dialog
	*   String title = The title of the dialog
	*   boolean modal = True to block user input to other to-level windows,
	*     false otherwise.
	* Data Returned:
	*   None
	* Method Description:
	*   Creates a message dialog.
	**************************************************************************/
	public static void showMessageDialog(Frame owner, String message, String title, boolean modal)
	{
		showDialog(owner, message, title, modal, m_s_InfoIcon, OK_OPTION);
	}

	/**************************************************************************
	* Method Name:
	*   showWarningDialog
	* Arguments Passed:
	*   Frame owner = The owner of the dialog.  Null if none
	*   String message = The message to display in the dialog
	* Data Returned:
	*   None
	* Method Description:
	*   Creates a warning dialog with the title 'Warning' and blocks user
	*   interaction to top-level windows.
	**************************************************************************/
	public static void showWarningDialog(Frame owner, String message)
	{
		showDialog(owner, message, "Warning", true, m_s_WarningIcon, OK_OPTION);
	}
	
	/**************************************************************************
	* Method Name:
	*   showWarningDialog
	* Arguments Passed:
	*   Frame owner = The owner of the dialog.  Null if none
	*   String message = The message to display in the dialog
	*   String title = The title of the dialog
	* Data Returned:
	*   None
	* Method Description:
	*   Creates a warning dialog that blocks user interaction to top-level 
	*   windows.
	**************************************************************************/
	public static void showWarningDialog(Frame owner, String message, String title)
	{
		showDialog(owner, message, title, true, m_s_WarningIcon, OK_OPTION);
	}
	
	/**************************************************************************
	* Method Name:
	*   showWarningDialog
	* Arguments Passed:
	*   Frame owner = The owner of the dialog.  Null if none
	*   String message = The message to display in the dialog
	*   boolean modal = True to block user input to other to-level windows,
	*     false otherwise.
	* Data Returned:
	*   None
	* Method Description:
	*   Creates a warning dialog with the title 'Warning'.
	**************************************************************************/
	public static void showWarningDialog(Frame owner, String message, boolean modal)
	{
		showDialog(owner, message, "Warning", modal, m_s_WarningIcon, OK_OPTION);
	}
	
	/**************************************************************************
	* Method Name:
	*   showWarningDialog
	* Arguments Passed:
	*   Frame owner = The owner of the dialog.  Null if none
	*   String message = The message to display in the dialog
	*   String title = The title of the dialog
	*   boolean modal = True to block user input to other to-level windows,
	*     false otherwise.
	* Data Returned:
	*   None
	* Method Description:
	*   Creates a warning dialog.
	**************************************************************************/
	public static void showWarningDialog(Frame owner, String message, String title, boolean modal)
	{
		showDialog(owner, message, title, modal, m_s_WarningIcon, OK_OPTION);
	}
	
	/**************************************************************************
	* Method Name:
	*   showQuestionDialog
	* Arguments Passed:
	*   Frame owner = The owner of the dialog.  Null if none
	*   String message = The message to display in the dialog
	*   String title = The title of the dialog
	* Data Returned:
	*   int = YES_OPTION if the yes button was pressed, NO_OPTION otherwise
	* Method Description:
	*   Creates a question dialog that blocks user interaction to top-level 
	*   windows.
	**************************************************************************/
	public static int showQuestionDialog(Frame owner, String message, String title)
	{
		return showDialog(owner, message, title, true, m_s_QuestionIcon, YES_NO_OPTION);
	}
	
	/**************************************************************************
	* Method Name:
	*   showQuestionDialog
	* Arguments Passed:
	*   Frame owner = The owner of the dialog.  Null if none
	*   String message = The message to display in the dialog
	*   boolean modal = True to block user input to other to-level windows,
	*     false otherwise.
	* Data Returned:
	*   int = YES_OPTION if the yes button was pressed, NO_OPTION otherwise
	* Method Description:
	*   Creates a question dialog with the title 'Warning' and blocks user 
	*   interaction to top-level windows.
	**************************************************************************/
	public static int showQuestionDialog(Frame owner, String message)
	{
		return showDialog(owner, message, "Question", true, m_s_QuestionIcon, YES_NO_OPTION);
	}
	
	/**************************************************************************
	* Method Name:
	*   showDialog
	* Arguments Passed:
	*   Frame owner = The owner of the dialog.  Null if none
	*   String message = The message to display in the dialog
	*   boolean modal = True to block user input to other to-level windows,
	*     false otherwise.
	*   String iconFile = Icon to show on the dialog.
	*   int dialogType = The dialog type.  See Method Description for 
	*     information on valid dialog types.
	* Data Returned:
	*   int = value that the dialog will return.
	* Method Description:
	*   Creates a question dialog.  Return values and dialog types available...
	*       MyDialog.OK_OPTION
	*       MyDialog.YES_NO_OPTION;
	*       MyDialog.YES_OPTION;
	*       MyDialog.NO_OPTION;
	**************************************************************************/
	public static int showDialog(Frame owner, String message, String title, boolean modal, String iconFile, int dialogType)
	{
		MyDialog dialog = new MyDialog(owner, message, title, modal, iconFile, dialogType);
		dialog.setVisible(true);

		return dialog.getOption();
	}
	
	/**************************************************************************
	* Method Name:
	*   getOption
	* Arguments Passed:
	*   None
	* Data Returned:
	*   int = value that the dialog will return.  
	* Method Description:
	*   This method returns a int value whether or not the submit button
	*   was pressed.
    *     Options available...
	*       MyDialog.OK_OPTION
	*       MyDialog.YES_NO_OPTION;
	*       MyDialog.YES_OPTION;
	*       MyDialog.NO_OPTION;
	**************************************************************************/
	public int getOption()
	{
		return option;
	}
	
	/**************************************************************************
	* Method Name:
	*   setOption
	* Arguments Passed:
	*   int option = value that the dialog will return.
	* Data Returned:
	*   None
	* Method Description:
	*   This method is called is called when it is needed to change the return
	*   value of the dialog box.  It is set to private to only allow members
	*   of the class change it.
    *     Options available...
	*       MyDialog.OK_OPTION
	*       MyDialog.YES_NO_OPTION;
	*       MyDialog.YES_OPTION;
	*       MyDialog.NO_OPTION;
	**************************************************************************/
	private synchronized void setOption(int option)
	{
		this.option = option;
	}
}