/******************************************************************************
* File name:
*   RecipientDialog.java
* Author:
*   Barham, Ray
* Creation Date:
*   06/19/07 
* Developed on:
*   WinXP
* Developed with:
*   Java(TM) SE Development Kit 6
*   JCreator 4.00.028
*
* Description:
*   This class is used to display a dialog box for the MyChat program.  It
*   handles the adding, editing, and deleteing of recipients for the 
*   recipient DefaultListModel.  It can display a form to fill out or add 
*   silently.  It was made to update the recipients in the MyChat program 
*   without hulting the receiving of messages (see Dialog.ModalityType for
*   more info).
*
*   An instance of this class can only be created using the provided static
*   methods.  This is a singleton class.  Only one instance of this class is 
*   allowed to be created at any given time.  That is, if the dialog is 
*   visible, you cannot open another.  
******************************************************************************/

/******************************************************************************
* Import statements
******************************************************************************/
// Util import statements
import java.util.Enumeration;

// AWT import statements
import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;

// Swing import statements
import javax.swing.DefaultListModel;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

final public class RecipientDialog extends JDialog implements ActionListener, KeyListener
{
	/**************************************************************************
	* Data Members:
	*  JTextField m_jtf_Name = The text field for the user name.
	*  JTextField m_jtf_Host = The text field for the host name (the computer to connect to).
	*  JButton m_jb_SubmitButton = The submit button.
	*  JButton m_jb_CancelButton = The cancel button.
	*  DefaultListModel m_dlm_Recipients = List model containing Contact objects for all the recipients.
	*  Contact m_c_Contact = The recipient that is currently being processed.
	*
	*  static boolean m_b_IsInstance = If there is an instance of this object already created.
	**************************************************************************/
	private JTextField m_jtf_Name;
	private JTextField m_jtf_Host;
	private JButton m_jb_SubmitButton;
	private JButton m_jb_CancelButton;
	private DefaultListModel m_dlm_Recipients;
	private Contact m_c_Contact;
	
	private static boolean m_b_IsInstance = false;
	
	/**************************************************************************
	* Method Name:
	*   RecipientDialog
	* Arguments Passed:
	*   Dialog owner = The owner of the dialog
	*   String title = The title of the dialog
	*   boolean modal = False for a modeless dialog, true otherwise.
	*   Contact contact = The recipient to process
	*   DefaultListModel recipients = The list model to add/edit the recipient 
	*     from.
	* Data Returned:
	*   None
	* Method Description:
	*   Private constructor used in constructing a recipient dialog with a 
	*   Dialog object as the owner.  Must use one of the provided static
	*   methods to construct a RecipientDialog object.
	**************************************************************************/
	private RecipientDialog(Dialog owner, String title, boolean modal, Contact contact, DefaultListModel recipients)
	{
		super(owner, title, modal);
		createDialog(contact, recipients);
	}
	
	/**************************************************************************
	* Method Name:
	*   RecipientDialog
	* Arguments Passed:
	*   Frame owner = The owner of the dialog
	*   String title = The title of the dialog
	*   boolean modal = False for a modeless dialog, true otherwise.
	*   Contact contact = The recipient to process
	*   DefaultListModel recipients = The list mode to add/edit the recipient 
	*     from.
	* Data Returned:
	*   None
	* Method Description:
	*   Private constructor used in constructing a recipient dialog with a 
	*   Frame object as the owner.  Must use one of the provided static
	*   methods to construct a RecipientDialog object.
	**************************************************************************/
	private RecipientDialog(Frame owner, String title, boolean modal, Contact contact, DefaultListModel recipients)
	{
		super(owner, title, modal);
		createDialog(contact, recipients);
	}

	/**************************************************************************
	* Method Name:
	*   createDialog
	* Arguments Passed:
	*   Contact contact = The recipient to process
	*   DefaultListModel recipients = The list model to add/edit the recipient 
	*     from.
	* Data Returned:
	*   None
	* Method Description:
	*   Private method used in aiding in the constructing of a recipient 
	*   dialog.  Provides all the work needed in creating the dialog.
	**************************************************************************/
	private void createDialog(Contact contact, DefaultListModel recipients)
	{
		m_c_Contact = contact;
		m_dlm_Recipients = recipients;
		
		/*
		 *  Setup the content pane
		 */
		JPanel contentPane = new JPanel(new BorderLayout());
		
		/*
		 *  Create and setup the text fields for the dialog box
		 */
		m_jtf_Name = new JTextField();
		m_jtf_Name.setText(m_c_Contact.getName());
		m_jtf_Name.addActionListener(this);

		m_jtf_Host = new JTextField();
		m_jtf_Host.setText(m_c_Contact.getHost());
		m_jtf_Host.addActionListener(this);
		
		/*
		 *  Create and setup the buttons for the dialog box
		 */
		m_jb_SubmitButton = new JButton("Submit");
		m_jb_SubmitButton.addActionListener(this);
		m_jb_SubmitButton.addKeyListener(this);
		
		m_jb_CancelButton = new JButton("Cancel");
		m_jb_CancelButton.addActionListener(this);
		m_jb_CancelButton.addKeyListener(this);

		/*
		 *  Create, setup, and add components to the user input panel
		 */
		JPanel jpInputPanel = new JPanel(new GridLayout(2, 2));
		jpInputPanel.add(new JLabel("User Name: "));
		jpInputPanel.add(m_jtf_Name);
		jpInputPanel.add(new JLabel("Computer Name/IP: "));
		jpInputPanel.add(m_jtf_Host);
		
		//Note: createEmptyBorder(int top, int left, int bottom, int right)
		jpInputPanel.setBorder(BorderFactory.createEmptyBorder(3, 3, 0, 3)); 
		
		/*
		 *  Create, setup, and add components to the button panel
		 */
		JPanel jpButtonPanel = new JPanel(new GridLayout(1,2));
		jpButtonPanel.add(m_jb_SubmitButton);
		jpButtonPanel.add(m_jb_CancelButton);
		
		/*
		 *  Add the panels to the content pane
		 */
		contentPane.add(jpInputPanel, BorderLayout.CENTER);
		contentPane.add(jpButtonPanel, BorderLayout.SOUTH);
		
		/*
		 *  Setup the dialog
		 */
		setContentPane(contentPane);
		setResizable(false);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		WindowSetup.centerWindow(this, 260, 150);
		pack();
		addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				m_b_IsInstance = false;
			}
		});
		
		m_b_IsInstance = true;
	}
	
	/**************************************************************************
	* Method Name:
	*   actionPerformed
	* Arguments Passed:
	*   ActionEvent event = An object that refrences a performed event
	* Data Returned:
	*   None
	* Method Description:
	*   This method handles ActionListener events.
	**************************************************************************/
	public void actionPerformed(ActionEvent event)
	{
		/*
		 *  If the cancel button was pressed
		 */
		if(event.getSource() == m_jb_CancelButton)
			cancelButtonPress();
		
		/*
		 *  If the submit button was pressed or if enter was pressed in the name
		 *  or computer name fields.
		 */
		else if(event.getSource() == m_jb_SubmitButton || event.getSource() == m_jtf_Name || event.getSource() == m_jtf_Host)
			submitButtonPress();
	}

	/**************************************************************************
	* Method Name:
	*   keyPressed
	* Arguments Passed:
	*   KeyEvent event = <see the KeyListener interface>
	* Data Returned:
	*   None
	* Method Description:
	*   This method handles key pressed events.  This is called if the user
	*   presses the enter or escape key on any of the buttons.
	**************************************************************************/
	public void keyPressed(KeyEvent event)
	{
		/*
		 *  If the enter was pressed
		 */
		if(event.getKeyCode() == KeyEvent.VK_ENTER)
		{
			if(event.getSource() != m_jb_CancelButton)
				submitButtonPress();

			else
				cancelButtonPress();
		}
		
		/*
		 *  If the escape was pressed
		 */
		else if(event.getKeyCode() == KeyEvent.VK_ESCAPE)
			cancelButtonPress();
	}
	
	/**************************************************************************
	* Not used but required to implement.
	**************************************************************************/
	public void keyTyped(KeyEvent event){}
	
	/**************************************************************************
	* Not used but required to implement.
	**************************************************************************/
	public void keyReleased(KeyEvent event){}

	/**************************************************************************
	* Method Name:
	*   submitButtonPress
	* Arguments Passed:
	*   None
	* Data Returned:
	*   None
	* Method Description:
	*   Private method called if the submit button was pressed.
	**************************************************************************/
	private void submitButtonPress()
	{
		/*
		 *  Ensure that the user name and computer name fields are not blank.
		 */
		if(m_jtf_Name.getText().isEmpty())
			MyDialog.showMessageDialog(null, "Please enter a user name.", true);

		else if(m_jtf_Host.getText().isEmpty())
			MyDialog.showMessageDialog(null, "Please enter a computer name/IP.", true);

		else
		{
			String oldName = m_c_Contact.getName();
			String oldHost = m_c_Contact.getHost();
			
			m_c_Contact.setName(m_jtf_Name.getText());
			m_c_Contact.setHost(m_jtf_Host.getText());
			
			/*
			 *  Process the contact, adding or updating it as needed.
			 */
			if(!processContact(m_c_Contact, m_dlm_Recipients))
			{
				m_c_Contact.setName(oldName);
				m_c_Contact.setHost(oldHost);
				
				return;
			}
	
			m_b_IsInstance = false;
			setVisible(false);
		}
	}

	/**************************************************************************
	* Method Name:
	*   cancelButtonPress
	* Arguments Passed:
	*   None
	* Data Returned:
	*   None
	* Method Description:
	*   Private method called if the cancel button was pressed.
	**************************************************************************/
	private void cancelButtonPress()
	{
		m_b_IsInstance = false;
		setVisible(false);
	}
	
	/**************************************************************************
	* Method Name:
	*   processContact
	* Arguments Passed:
	*   Contact contact = The recipient that is currently being processed.
	*   DefaultListModel recipients = List mode containing Contact objects for 
	*     all the recipients.
	* Data Returned:
	*   boolean
	* Method Description:
	*   Private method used in adding/updating a recipient in the provided
	*   DefaultListModel.
	**************************************************************************/
	private static boolean processContact(Contact contact, DefaultListModel recipients)
	{
		if(recipients != null)
		{
			/*
			 *  Check to see if the recipient already exists.  That is, if we are adding or updating.
			 */
			if(!recipients.contains(contact))
			{
				/*
				 *  Cycle through the recipient list model and check to see if there is a matching host name.
				 */
				for(Enumeration enumeration = recipients.elements(); enumeration.hasMoreElements();)
				{
					Contact existingContact = (Contact)enumeration.nextElement();					
		
					if(contact.getHost().compareToIgnoreCase(existingContact.getHost()) == 0)
					{
						MyDialog.showMessageDialog(null, "Someone on that computer is already in your recipient list.",	true);
						
						return false;
					}
				}
				
				// Store the new recipient in the list model.
				recipients.addElement(contact);
				
				return true;
			}
			
			else
			{
				/*
				 *  Cycle through the recipient list model and check to see if there is a matching host name.
				 */
				for(Enumeration enumeration = recipients.elements(); enumeration.hasMoreElements();)
				{
					Contact existingContact = (Contact)enumeration.nextElement();					
		
					if(contact.getHost().compareToIgnoreCase(existingContact.getHost()) == 0)
					{
						if(contact != existingContact)
						{
							MyDialog.showMessageDialog(null, "Someone on that computer is already in your recipient list.",	true);
							
							return false;
						}
					}
				}

				// Cause the recipient list model to send a ListDataEvent to the
				// contentsChanged(ListDataEvent e) method.
				recipients.setElementAt(contact, recipients.indexOf(contact));
				
				return true;
			}
		}
		
		return true;
	}

	/**************************************************************************
	* Method Name:
	*   showDialog
	* Arguments Passed:
	*   Dialog owner = The owner of the dialog
	*   String title = The title of the dialog
	*   boolean modal = False for a modeless dialog, true otherwise.
	*   Contact contact = The recipient to process
	* Data Returned:
	*   None
	* Method Description:
	*   Static method used in constructing a dialog box that changes the
	*   passed Contact object and that as a Dialog owner
	**************************************************************************/
	public static void showDialog(Dialog owner, String title, boolean modal, Contact contact)
	{
		if(!m_b_IsInstance)
		{
			RecipientDialog dialog = new RecipientDialog(owner, title, modal, contact, null);
			dialog.setVisible(true);
		}
	}
	
	/**************************************************************************
	* Method Name:
	*   showDialog
	* Arguments Passed:
	*   Frame owner = The owner of the dialog
	*   String title = The title of the dialog
	*   boolean modal = False for a modeless dialog, true otherwise.
	*   Contact contact = The recipient to process
	* Data Returned:
	*   None
	* Method Description:
	*   Static method used in constructing a dialog box that changes the
	*   passed Contact object and that as a Frame owner
	**************************************************************************/
	public static void showDialog(Frame owner, String title, boolean modal, Contact contact)
	{
		if(!m_b_IsInstance)
		{
			RecipientDialog dialog = new RecipientDialog(owner, title, modal, contact, null);
			dialog.setVisible(true);
		}
	}
	
	/**************************************************************************
	* Method Name:
	*   addRecipient
	* Arguments Passed:
	*   Contact contact = The recipient that needs to be added.
	*   DefaultListModel recipients = List model that the recipient will be 
	*     added to.
	* Data Returned:
	*   None
	* Method Description:
	*   Static method used in adding a recipient to the provided 
	*   DefaultListModel.
	**************************************************************************/
	public static void addRecipient(Contact contact, DefaultListModel recipients)
	{
		processContact(contact, recipients);
	}

	/**************************************************************************
	* Method Name:
	*   addRecipient
	* Arguments Passed:
	*   Dialog owner = The owner of the dialog
	*   boolean modal = False for a modeless dialog, true otherwise.
	*   DefaultListModel recipients = List model that the new recipient will be 
	*     added to.
	* Data Returned:
	*   None
	* Method Description:
	*   Static method used in constructing a dialog box with a Dialog owner 
	*   that will add a recipient to the provided DefaultListModel.
	**************************************************************************/
	public static void addRecipient(Dialog owner, boolean modal, DefaultListModel recipients)
	{
		if(!m_b_IsInstance)
		{
			RecipientDialog dialog = new RecipientDialog(owner, "Add Recipient", modal, new Contact(), recipients);
			dialog.setVisible(true);
		}
	}

	/**************************************************************************
	* Method Name:
	*   addRecipient
	* Arguments Passed:
	*   Frame owner = The owner of the dialog
	*   boolean modal = False for a modeless dialog, true otherwise.
	*   DefaultListModel recipients = List model that the new recipient will be 
	*     added to.
	* Data Returned:
	*   None
	* Method Description:
	*   Static method used in constructing a dialog box with a Frame owner 
	*   that will add a recipient to the provided DefaultListModel.
	**************************************************************************/
	public static void addRecipient(Frame owner, boolean modal, DefaultListModel recipients)
	{
		if(!m_b_IsInstance)
		{
			RecipientDialog dialog = new RecipientDialog(owner, "Add Recipient", modal, new Contact(), recipients);
			dialog.setVisible(true);
		}
	}

	/**************************************************************************
	* Method Name:
	*   editRecipient
	* Arguments Passed:
	*   Dialog owner = The owner of the dialog
	*   boolean modal = False for a modeless dialog, true otherwise.
	*   Contact contact = The recipient to edit
	*   DefaultListModel recipients = List model that the recipient is located 
	*     in.
	* Data Returned:
	*   None
	* Method Description:
	*   Static method used in constructing a dialog box with a Dialog owner 
	*   that will edit a provided recipient (or contact).  The DefaultListModel
	*   is used for comparing the edit recipient with those already existing.
	**************************************************************************/
	public static void editRecipient(Dialog owner, boolean modal, Contact contact, DefaultListModel recipients)
	{
		if(!m_b_IsInstance)
		{
			RecipientDialog dialog = new RecipientDialog(owner, "Edit Recipient", modal, contact, recipients);
			dialog.setVisible(true);
		}
	}
	
	/**************************************************************************
	* Method Name:
	*   editRecipient
	* Arguments Passed:
	*   Frame owner = The owner of the dialog
	*   boolean modal = False for a modeless dialog, true otherwise.
	*   Contact contact = The recipient to edit
	*   DefaultListModel recipients = List model that the recipient is located 
	*     in.
	* Data Returned:
	*   None
	* Method Description:
	*   Static method used in constructing a dialog box with a Dialog owner 
	*   that will edit a provided recipient (or contact).  The DefaultListModel
	*   is used for comparing the edit recipient with those already existing.
	**************************************************************************/
	public static void editRecipient(Frame owner, boolean modal, Contact contact, DefaultListModel recipients)
	{
		if(!m_b_IsInstance)
		{
			RecipientDialog dialog = new RecipientDialog(owner, "Edit Recipient", modal, contact, recipients);
			dialog.setVisible(true);
		}
	}
}